/*  libhrel - Holyrel library

    Copyright (C) 2005

    Chong Kai Xiong <descender@phreaker.net>
    Milosz Derezynski <internalerror@gmail.com>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the
    Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA  02111-1307  USA.
*/

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <libhrel/types.h>
#include <libhrel/relation.h>
#include <libhrel/gtk_relation_store.h>
#include <stdio.h>

typedef struct _Track Track;

struct _Track
{
  gchar *name;
  gchar *uri;
  gchar *artist;
  guint  count;
};

static const gchar *column_titles[] =
{
 "Name",
 "Artist",
 "Play count",
 "URI"
};

static HRelation *tracks;

static void
tracks_init (void)
{
  tracks = h_relation_base_new ("track",
                                "track has name, uri, artist, play count",
                                "name"  , G_TYPE_STRING,
                                "uri"   , G_TYPE_STRING,
                                "artist", G_TYPE_STRING,
                                "count" , G_TYPE_INT,
                                NULL,
                                "uri",
                                NULL);
}

static void
tracks_add_track (Track *track)
{
  h_relation_base_insert_tuplev ((HRelationBase *) tracks,
                                 "name"  , track->name,
                                 "uri"   , track->uri,
                                 "artist", track->artist,
                                 "count" , track->count);
}

static void
tracks_populate (void)
{
  static Track track_data[] =
  {
    {
      "Vincent",
      "file:///home/descender/Documents/music/Don McLean/Vincent.ogg",
      "Don McLean",
      1
    },
    {
      "Before I was gone",
      "file:///home/descender/Documents/music/Ehren Starks/Before I was gone.ogg",
      "Ehren Starks",
      3
    },

    {
      "New Year's Love",
      "file:///home/descender/Documents/music/Ehren Starks/New Year's Love.ogg",
      "Ehren Starks",
      5
    },

    {
      "Love, Me",
      "file:///home/descender/Documents/music/Collin Raye - Love Me.ogg",
      "Collin Raye",
      4
    },
    {
      "Eternal Snow",
      "file:///home/descender/Documents/music/soundtracks/Full Moon wo Sagashite/Eternal Snow.mp3",
      "Myco",
      6
    }
  };

  gint i;

  for (i = 0; i < G_N_ELEMENTS (track_data); i++)
    tracks_add_track (&track_data[i]);
}

static GtkWidget *
track_view_new (void)
{
  GtkTreeViewColumn *tree_view_column;
  GtkCellRenderer   *cell;
  GtkWidget         *window, *vbox, *tree_view;
  HGtkRelationStore *store;
  gint               n;

  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (window), "HGtkRelationStore demostration");

  vbox = gtk_vbox_new (FALSE, 1);
  gtk_container_add (GTK_CONTAINER (window), vbox);

  tree_view = gtk_tree_view_new ();
  gtk_box_pack_start (GTK_BOX (vbox), tree_view, TRUE, TRUE, 4);

  gtk_tree_view_set_headers_clickable (GTK_TREE_VIEW (tree_view), TRUE);

  for (n = 0; n < h_relation_get_degree (tracks); n++)
    {
      tree_view_column = gtk_tree_view_column_new ();

      cell = gtk_cell_renderer_text_new ();

      gtk_tree_view_column_pack_start (tree_view_column, cell, TRUE);
      gtk_tree_view_column_set_attributes (tree_view_column, cell, "text", n, NULL);
      gtk_tree_view_column_set_title (tree_view_column, column_titles[n]);
      gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), tree_view_column);
    }

  store = h_gtk_relation_store_new (tracks, "name", "artist", "count", "uri", NULL);
  gtk_tree_view_set_model (GTK_TREE_VIEW (tree_view), GTK_TREE_MODEL (store));

  gtk_widget_show_all (vbox);

  return window;
}

static gboolean
on_window_delete_event (GtkWidget *widget,
                        GdkEvent  *event,
                        gpointer   data)
{
  gtk_main_quit ();

  return TRUE;
}

gint
main (gint    n_args,
      gchar **args)
{
  GtkWidget *window;

  gtk_init (&n_args, &args);

  h_type_init ();

  tracks_init ();
  tracks_populate ();

  window = track_view_new ();

  g_signal_connect (G_OBJECT (window), "delete-event",
                    G_CALLBACK (on_window_delete_event),
                    NULL);

  gtk_widget_show (window);

  gtk_main ();

  h_relation_unref (tracks);

  return 0;
}
