/* -*- Mode: C; -*- */

/*  libhrel - Holyrel library

    Copyright (C) 2005 Chong Kai Xiong <descender@phreaker.net>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the
    Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA  02111-1307  USA.
*/

%name-prefix="rl_"

%{

#include <stdio.h>
#include <glib.h>

extern gint     rl_lex              (void);
extern gboolean rl_set_input_file   (const gchar *filename);
extern gboolean rl_set_input_string (const gchar *filename,
                                     gsize        length);

static void     rl_error            (const gchar *msssage);

%}

%union
{
  GString *string;
  gint     integer;
  gdouble  real;
}

%token <string>  RL_TOKEN_IDENTIFIER
%token <string>  RL_TOKEN_STRING
%token <integer> RL_TOKEN_INTEGER
%token <real>    RL_TOKEN_REAL

%%

/*
 * expression = identifier
 *            | string
 *            | integer
 *            | real number
 *            | "'" expression
 *            | '(' [ { expression } ] ')'
 *            ;
 */
expr: RL_TOKEN_IDENTIFIER
    | RL_TOKEN_STRING
    | RL_TOKEN_INTEGER
    | RL_TOKEN_REAL
    | '\'' expr
    | '(' expr_list_opt ')'
;

expr_list_opt: /* nothing */
             | expr_list
;

expr_list: expr
         | expr expr_list
;

%%

#include <string.h>

static void
rl_error (const gchar *message)
{
  g_message ("Parser error: %s", message);
}

gboolean
rl_parse_string (const gchar *string,
                 gsize        length)
{
  gint result = 1;

  g_return_val_if_fail (string != NULL, FALSE);

  rl_set_input_string (string, length);
  result = rl_parse ();

  return (result == 0);
}

gboolean
rl_parse_file (const gchar *filename)
{
  gint result = 1;

  g_return_val_if_fail (filename != NULL, FALSE);

  rl_set_input_file (filename);
  result = rl_parse ();

  return (result == 0);
}
