/* -*- mode: C; -*- */

/*  libhrel - Holyrel library

    Copyright (C) 2005 Chong Kai Xiong <descender@phreaker.net>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the
    Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA  02111-1307  USA.
*/

%option prefix="rl_"

%option noyywrap
%option nounput

%{

#include <stdio.h>
#include <stdlib.h>
#include <glib.h>
#include "rl_parser.h"

%}

%x quoted_string

DIGIT         [0-9]
LETTER        [a-zA-Z]
ALPHANUM      {DIGIT}|{LETTER}
OPERATORS     "+"|"-"|"*"|"/"|"="|"<"|">"|"!"|"&"|"|"|"^"|"_"|"@"|"$"|"%"|"{"|"}"
SIGN          "+"|"-"
INTEGER       {SIGN}?{DIGIT}+
REAL          {INTEGER}("."{DIGIT}*)?([Ee]{INTEGER})?

%%

{INTEGER} {
  /* Integer */
  rl_lval.integer = atoi (rl_text);
  return RL_TOKEN_INTEGER;
}

{REAL} {
  /* Real number */
  rl_lval.real = atof (rl_text);
  return RL_TOKEN_REAL;
}

({ALPHANUM}|{OPERATORS})+ {
  /* Identifier */
  g_string_assign (rl_lval.string, rl_text);
  return RL_TOKEN_IDENTIFIER;
}

";".*$      /* Ignore comment */
[ \t]+      /* Ignore whitespace */
\n          /* TODO: count lines */

"'"|"("|")"|","|"`" {
  return yytext[0];
}

\" {
  BEGIN(quoted_string);
  g_string_assign (rl_lval.string, "");
}

<quoted_string>[^\"\n]* {
  g_string_append (rl_lval.string, rl_text);
}

<quoted_string>\" {
  BEGIN(INITIAL);
  return RL_TOKEN_STRING;
}

%%

gboolean
rl_set_input_string (const gchar *string,
                     gsize        length)
{
  return FALSE;
}

gboolean
rl_set_input_file (const gchar *filename)
{
  return FALSE;
}
