#include <config.h>
#include <libhrel/types.h>
#include <libhrel/hash_index.h>

#include <glib.h>
#include <stdio.h>
#include <stdlib.h>

typedef struct
{
  const gchar *name;
  guint        age;
  guint        weight;
}
People;

static People people[] =
{
  {"Ally", 20, 50 },
  {"Beth", 19, 60 },
  {"Cath", 24, 45 },
  {"Dawn", 23, 42 },

  {"Ally", 20, 45 },
  {"Beth", 19, 60 },

  {"Cath", 23, 45 },
  {"Dawn", 22, 38 }
};

static void
print_entry (HTuple      *tuple,
             gint         number,
             const gchar *prefix)
{
  printf ("%s", prefix);
  h_tuple_print (tuple);
  printf ("-> %d\n", number);
}

gint
main (void)
{
  HHashIndex *index;
  HTuple     *tuple;
  gint        i;

  h_type_init ();

  index = h_hash_index_new ((GDestroyNotify) h_tuple_free,
                            NULL,
                            "name", "age", NULL);

  for (i = 0; i < G_N_ELEMENTS (people); i++)
    {
      tuple = h_tuple_new ();

      h_tuple_insert (tuple, "name"  , G_TYPE_STRING, (gpointer) people[i].name);
      h_tuple_insert (tuple, "age"   , G_TYPE_INT   , GINT_TO_POINTER (people[i].age));
      h_tuple_insert (tuple, "weight", G_TYPE_INT   , GINT_TO_POINTER (people[i].weight));

      h_hash_index_insert (index, tuple, GINT_TO_POINTER (i));
    }

  printf ("Index:\n");
  h_hash_index_foreach (index, (HIndexFunc) print_entry, "Tuple: ");

  h_hash_index_free (index);

  return EXIT_SUCCESS;
}
